@@ -1,30 +1,30 @@
 <?php
 /**
- * Base class for managing everything related to state:
+ * Horde_ActiveSync_State_Base::
  *
- *     Persistence of state data
- *     Generating delta between server and PIM
- *     Caching PING related state (hearbeat interval, folder list etc...)
+ * @license   http://www.horde.org/licenses/gpl GPLv2
+ *            NOTE: According to sec. 8 of the GENERAL PUBLIC LICENSE (GPL),
+ *            Version 2, the distribution of the Horde_ActiveSync module in or
+ *            to the United States of America is excluded from the scope of this
+ *            license.
+ * @copyright 2009-2013 Horde LLC (http://www.horde.org)
+ * @author    Michael J Rubinsky <mrubinsk@horde.org>
+ * @package   ActiveSync
+ */
+/**
+ * Base class for managing everything related to device state
  *
- * Copyright 2010-2012 Horde LLC (http://www.horde.org/)
- *
- * @author Michael J. Rubinsky <mrubinsk@horde.org>
- * @package ActiveSync
+ * @license   http://www.horde.org/licenses/gpl GPLv2
+ *            NOTE: According to sec. 8 of the GENERAL PUBLIC LICENSE (GPL),
+ *            Version 2, the distribution of the Horde_ActiveSync module in or
+ *            to the United States of America is excluded from the scope of this
+ *            license.
+ * @copyright 2009-2013 Horde LLC (http://www.horde.org)
+ * @author    Michael J Rubinsky <mrubinsk@horde.org>
+ * @package   ActiveSync
  */
 abstract class Horde_ActiveSync_State_Base
 {
-    /**
-     * Filtertype constants
-     */
-    const FILTERTYPE_ALL = 0;
-    const FILTERTYPE_1DAY = 1;
-    const FILTERTYPE_3DAYS = 2;
-    const FILTERTYPE_1WEEK = 3;
-    const FILTERTYPE_2WEEKS = 4;
-    const FILTERTYPE_1MONTH = 5;
-    const FILTERTYPE_3MONTHS = 6;
-    const FILTERTYPE_6MONTHS = 7;
-
     /**
      * Configuration parameters
      *
@@ -35,9 +35,10 @@ abstract class Horde_ActiveSync_State_Base
     /**
      * Caches the current state(s) in memory
      *
-     * @var array
+     * @var mixed Horde_ActiveSync_Folder_Base if request is not a FOLDERSYNC
+     *            otherwise an array containing all FOLDERSYNC state.
      */
-    protected $_stateCache;
+    protected $_folder;
 
     /**
      * The syncKey for the current request.
@@ -53,24 +54,16 @@ abstract class Horde_ActiveSync_State_Base
      */
     protected $_backend;
 
-    /**
-     * Cache for ping state
-     *
-     * @var array
-     */
-    protected $_pingState;
-
     /**
      * The collection array for the collection we are currently syncing.
      * Keys include:
-     *   'class'      - The collection class Contacts, Calendar etc...
-     *   'synckey'    - The current synckey
-     *   'newsynckey' - The new synckey sent back to the PIM
-     *   'id'         - Server folder id
-     *   'filtertype' - Filter
-     *   'conflict'   - Conflicts
-     *   'truncation' - Truncation
-     *
+     *   - class:       The collection class Contacts, Calendar etc...
+     *   - synckey:     The current synckey
+     *   - newsynckey:  The new synckey sent back to the PIM
+     *   - id:          Server folder id
+     *   - filtertype:  Filter
+     *   - conflict:    Conflicts
+     *   - truncation:  Truncation
      *
      * @var array
      */
@@ -84,16 +77,16 @@ abstract class Horde_ActiveSync_State_Base
     protected $_logger;
 
     /**
-     * The PIM device id. Needed for PING requests
+     * Device info structure. Contains the following properties:
+     *  'rwstatus'   - Device RemoteWipe status.
+     *  'deviceType' - The device type.
+     *  'userAgent'  - The device's userAgent string.
+     *  'id'         - Device id.
+     *  'user'       - The user associated with the current account.
+     *  'supported'  - The SUPPORTED response for the device's collections.
+     *  'policykey'  - The device's current POLICYKEY.
      *
-     * @var string
-     */
-    protected $_devId;
-
-    /**
-     * Device info cache
-     *
-     * @var object
+     * @var StdClass
      */
     protected $_deviceInfo;
 
@@ -101,12 +94,12 @@ abstract class Horde_ActiveSync_State_Base
      * Local cache for changes to *send* to PIM
      * (Will remain null until getChanges() is called)
      *
-     * @var
+     * @var array
      */
     protected $_changes;
 
     /**
-     * The type of request we are handling (if important).
+     * The type of request we are handling.
      *
      * @var string
      */
@@ -115,22 +108,24 @@ abstract class Horde_ActiveSync_State_Base
     /**
      * Const'r
      *
-     * @param array $collection  A collection array
-     * @param array $params  All configuration parameters, requirements.
-     *
-     * @return Horde_ActiveSync_State_Base
+     * @param array $params  All configuration parameters.
      */
-    public function __construct($params = array())
+    public function __construct(array $params = array())
     {
         $this->_params = $params;
         if (empty($params['logger'])) {
             $this->_logger = new Horde_Support_Stub();
+        } else {
+            $this->_logger = $params['logger'];
         }
     }
 
+    /**
+     * Destructor
+     */
     public function __destruct()
     {
-        unset ($this->_backend);
+        unset($this->_backend);
     }
 
     /**
@@ -175,9 +170,8 @@ abstract class Horde_ActiveSync_State_Base
      */
     public function getPolicyKey($devId)
     {
-        //@TODO - combine _devId and _deviceInfo
         /* See if we have it already */
-        if (empty($this->_deviceInfo) || $this->_devId != $devId) {
+        if (empty($this->_deviceInfo) || $this->_deviceInfo->id != $devId) {
             throw new Horde_ActiveSync_Exception('Device not loaded.');
         }
 
@@ -193,9 +187,8 @@ abstract class Horde_ActiveSync_State_Base
      */
     public function getDeviceRWStatus($devId)
     {
-        //@TODO - combine _devId and _deviceInfo
         /* See if we have it already */
-        if (empty($this->_deviceInfo) || $this->_devId != $devId) {
+        if (empty($this->_deviceInfo) || $this->_deviceInfo->id != $devId) {
             throw new Horde_ActiveSync_Exception('Device not loaded.');
         }
 
@@ -216,18 +209,6 @@ abstract class Horde_ActiveSync_State_Base
         $this->_backend = $backend;
     }
 
-    /**
-     * Initialize the state object
-     *
-     * @param array $collection  The collection array
-     *
-     * @return void
-     */
-    public function init($collection = array())
-    {
-        $this->_collection = $collection;
-    }
-
     /**
      * Set the logger instance for this object.
      *
@@ -238,19 +219,6 @@ abstract class Horde_ActiveSync_State_Base
         $this->_logger = $logger;
     }
 
-    /**
-     * Reset the device's PING state.
-     *
-     * @return void
-     */
-    public function resetPingState()
-    {
-        $this->_logger->debug('Resetting PING state');
-        $this->_pingState = array(
-            'lifetime' => 0,
-            'collections' => array());
-    }
-
     /**
      * Get the number of server changes.
      *
@@ -290,6 +258,23 @@ abstract class Horde_ActiveSync_State_Base
         }
     }
 
+    /**
+     * Return the counter for the specified syncKey.
+     *
+     * @param string $syncKey  The synckey to obtain the counter for.
+     *
+     * @return mixed integer|boolean  The increment counter or false if failed.
+     */
+    static public function getSyncKeyCounter($syncKey)
+    {
+       if (preg_match('/^s{0,1}\{([a-fA-F0-9-]+)\}([0-9]+)$/', $syncKey, $matches)) {
+            $n = $matches[2];
+            return $n;
+        }
+
+        return false;
+    }
+
    /**
     * Returns the timestamp of the earliest modification time to consider
     *
@@ -300,26 +285,26 @@ abstract class Horde_ActiveSync_State_Base
     static protected function _getCutOffDate($restrict)
     {
         switch($restrict) {
-        case self::FILTERTYPE_1DAY:
-            $back = 60 * 60 * 24;
+        case Horde_ActiveSync::FILTERTYPE_1DAY:
+            $back = 86400;
             break;
-        case self::FILTERTYPE_3DAYS:
-            $back = 60 * 60 * 24 * 3;
+        case Horde_ActiveSync::FILTERTYPE_3DAYS:
+            $back = 259200;
             break;
-        case self::FILTERTYPE_1WEEK:
-            $back = 60 * 60 * 24 * 7;
+        case Horde_ActiveSync::FILTERTYPE_1WEEK:
+            $back = 604800;
             break;
-        case self::FILTERTYPE_2WEEKS:
-            $back = 60 * 60 * 24 * 14;
+        case Horde_ActiveSync::FILTERTYPE_2WEEKS:
+            $back = 1209600;
             break;
-        case self::FILTERTYPE_1MONTH:
-            $back = 60 * 60 * 24 * 31;
+        case Horde_ActiveSync::FILTERTYPE_1MONTH:
+            $back = 2419200;
             break;
-        case self::FILTERTYPE_3MONTHS:
-            $back = 60 * 60 * 24 * 31 * 3;
+        case Horde_ActiveSync::FILTERTYPE_3MONTHS:
+            $back = 7257600;
             break;
-        case self::FILTERTYPE_6MONTHS:
-            $back = 60 * 60 * 24 * 31 * 6;
+        case Horde_ActiveSync::FILTERTYPE_6MONTHS:
+            $back = 14515200;
             break;
         default:
             break;
@@ -336,7 +321,7 @@ abstract class Horde_ActiveSync_State_Base
 
     /**
      * Helper function that performs the actual diff between PIM state and
-     * server state arrays.
+     * server state FOLDERSYNC arrays.
      *
      * @param array $old  The PIM state
      * @param array $new  The current server state
@@ -354,62 +339,53 @@ abstract class Horde_ActiveSync_State_Base
         $inew = 0;
         $iold = 0;
 
-        // Get changes by comparing our list of messages with
+        // Get changes by comparing our list of folders with
         // our previous state
         while (1) {
             $change = array();
-
             if ($iold >= count($old) || $inew >= count($new)) {
                 break;
             }
-
+            // If ids are the same, but mod is different, a folder was
+            // renamed on the client, but the server keeps it's id.
             if ($old[$iold]['id'] == $new[$inew]['id']) {
-                // Both messages are still available, compare flags and mod
-                if (isset($old[$iold]['flags']) && isset($new[$inew]['flags']) && $old[$iold]['flags'] != $new[$inew]['flags']) {
-                    // Flags changed
-                    $change['type'] = 'flags';
-                    $change['id'] = $new[$inew]['id'];
-                    $change['flags'] = $new[$inew]['flags'];
-                    $changes[] = $change;
-                }
-
+                // Both folders are still available compare mod
                 if ($old[$iold]['mod'] != $new[$inew]['mod']) {
-                    $change['type'] = 'change';
+                    $change['type'] = Horde_ActiveSync::CHANGE_TYPE_CHANGE;
+                    //$change['mod'] = $new[$inew]['mod'];
                     $change['id'] = $new[$inew]['id'];
                     $changes[] = $change;
                 }
-
                 $inew++;
                 $iold++;
             } else {
                 if ($old[$iold]['id'] > $new[$inew]['id']) {
-                    // Message in state seems to have disappeared (delete)
-                    $change['type'] = 'delete';
+                    // Messesge in device state has disappeared
+                    $change['type'] = Horde_ActiveSync::CHANGE_TYPE_DELETE;
                     $change['id'] = $old[$iold]['id'];
                     $changes[] = $change;
                     $iold++;
                 } else {
-                    // Message in new seems to be new (add)
-                    $change['type'] = 'change';
-                    $change['flags'] = Horde_ActiveSync::FLAG_NEWMESSAGE;
+                    // Message in $new is new
+                    $change['type'] = Horde_ActiveSync::CHANGE_TYPE_CHANGE;
                     $change['id'] = $new[$inew]['id'];
                     $changes[] = $change;
                     $inew++;
                 }
             }
         }
-
         while ($iold < count($old)) {
             // All data left in _syncstate have been deleted
-            $change['type'] = 'delete';
+            $change['type'] = Horde_ActiveSync::CHANGE_TYPE_DELETE;
             $change['id'] = $old[$iold]['id'];
             $changes[] = $change;
             $iold++;
         }
 
+        // New folders added on server.
         while ($inew < count($new)) {
             // All data left in new have been added
-            $change['type'] = 'change';
+            $change['type'] = Horde_ActiveSync::CHANGE_TYPE_CHANGE;
             $change['flags'] = Horde_ActiveSync::FLAG_NEWMESSAGE;
             $change['id'] = $new[$inew]['id'];
             $changes[] = $change;
@@ -432,30 +408,17 @@ abstract class Horde_ActiveSync_State_Base
     }
 
     /**
-     * Loads the initial state from storage for the specified syncKey and
-     * intializes the stateMachine for use.
+     * Loads the state from storage for the specified syncKey and intializes the
+     * object for use.
      *
-     * @param string $syncKey  The key for the state to load.
-     * @param string $type     Treat the loaded state data as this type of state.
-     * @param string $id       The collection id this represents
+     * @param array $collection  The collection array, if loading a collection.
+     * @param string $syncKey    The key for the state to load.
+     * @param string $type       Treat the loaded state data as this type of state.
+     * @param string $id         The collection id this represents
      *
      * @return array The state array
      */
-    abstract public function loadState($syncKey, $type = null, $id = '');
-
-    /**
-     * Load/initialize the ping state for the specified device.
-     *
-     * @param object $device
-     */
-    abstract public function initPingState($device);
-
-    /**
-     * Load the ping state for the given device id
-     *
-     * @param string $devid  The device id.
-     */
-    abstract public function loadPingCollectionState($devid);
+    abstract public function loadState(array $collection, $syncKey, $type = null, $id = null);
 
     /**
      * Get the list of known folders for the specified syncState
@@ -472,48 +435,38 @@ abstract class Horde_ActiveSync_State_Base
     /**
      * Update the state to reflect changes
      *
-     * @param string $type     The type of change (change, delete, flags)
-     * @param array $change    A stat/change hash describing the change
-     * @param integer $origin  Flag to indicate the origin of the change.
-     * @param string $user     The current synch user
+     * @param string $type      The type of change (change, delete, flags or
+     *                          foldersync)
+     * @param array $change     A stat/change hash describing the change.
+     *  Contains:
+     *    - id:      The message uid the change applies to
+     *    - parent:  The parent of the message, normally the folder id.
+     *    - flags:   If this is a flag change, the state of the read flag.
+     *    - mod:     The modtime of this change for collections that use it.
      *
-     * @return void
+     * @param integer $origin   Flag to indicate the origin of the change:
+     *    Horde_ActiveSync::CHANGE_ORIGIN_NA  - Not applicapble/not important
+     *    Horde_ActiveSync::CHANGE_ORIGIN_PIM - Change originated from PIM
+     *
+     * @param string $user      The current sync user, only needed if change
+     *                          origin is CHANGE_ORIGIN_PIM
+     * @param string $clientid  PIM clientid sent when adding a new message
      */
-    abstract public function updateState($type, array $change,
-                                         $origin = Horde_ActiveSync::CHANGE_ORIGIN_NA,
-                                         $user = null);
-
-    /**
-     * Save folder data for a specific device. This is needed for BC with older
-     * activesync versions that use GETHIERARCHY requests to get the folder info
-     * instead of maintaining the folder state with FOLDERSYNC requests.
-     *
-     * @param object $device  The device object
-     * @param array $folders  The folder data
-     *
-     * @return boolean
-     * @throws Horde_ActiveSync_Exception
-     */
-    abstract public function setFolderData($device, $folders);
-
-    /**
-     * Get the folder data for a specific device
-     *
-     * @param object $device  The device object
-     * @param string $class   The folder class to fetch (Calendar, Contacts etc.)
-     *
-     * @return mixed  Either an array of folder data || false
-     */
-    abstract public function getFolderData($device, $class);
+    abstract public function updateState(
+        $type, array $change, $origin = Horde_ActiveSync::CHANGE_ORIGIN_NA,
+        $user = null, $clientid = '');
 
     /**
      * Get all items that have changed since the last sync time
      *
-     * @param integer $flags
+     * @param array $options  An options array:
+     *      - ping:  (boolean)  Only detect if there is a change, do not build
+     *                          any messages.
+     *               DEFAULT: false (Build full change array).
      *
      * @return array
      */
-    abstract public function getChanges($flags = 0);
+    abstract public function getChanges(array $options = array());
 
     /**
      * Determines if the server version of the message represented by $stat
@@ -541,19 +494,18 @@ abstract class Horde_ActiveSync_State_Base
      * cause all devices that support provisioning to be reprovisioned.
      *
      * @throws Horde_ActiveSync_Exception
-     *
      */
     abstract public function resetAllPolicyKeys();
 
     /**
      * Set a new remotewipe status for the device
      *
-     * @param string $devid
-     * @param string $status
+     * @param string $devId    The device id.
+     * @param string $status   A Horde_ActiveSync::RWSTATUS_* constant.
      *
-     * @return boolean
+     * @throws Horde_ActiveSync_Exception
      */
-    abstract public function setDeviceRWStatus($devid, $status);
+    abstract public function setDeviceRWStatus($devId, $status);
 
     /**
      * Obtain the device object.
@@ -563,7 +515,7 @@ abstract class Horde_ActiveSync_State_Base
      *
      * @return StdClass
      */
-    abstract public function loadDeviceInfo($device, $user);
+    abstract public function loadDeviceInfo($device, $user = null);
 
     /**
      * Check that a given device id is known to the server. This is regardless
@@ -579,36 +531,34 @@ abstract class Horde_ActiveSync_State_Base
     /**
      * Set new device info
      *
-     * @param object $device  The device information
-     *
-     * @return boolean
+     * @param object $data  The device information
      */
     abstract public function setDeviceInfo($data);
 
+    /**
+     * Set the device's properties as sent by a SETTINGS request.
+     *
+     * @param array $data       The device settings
+     * @param string $deviceId  The device id.
+     *
+     * @throws Horde_ActiveSync_Exception
+     */
+    abstract public function setDeviceProperties(array $data, $deviceId);
+
     /**
      * Explicitly remove a state from storage.
      *
-     * @param string $synckey  The specific state to remove
-     * @param string $devId    Remove all state for this device (ignores synckey)
+     * @param array $options  An options array containing:
+     *   - synckey: (string)  Remove only the state associated with this synckey.
+     *   - devId: (string)  Remove all information for this device.
+     *   - user: (string)  When removing device info, restrict to removing data
+     *                    for this user only.
+     *   - id: (string)  When removing device state, restrict ro removing data
+     *                   only for this collection.
      *
      * @throws Horde_ActiveSyncException
      */
-    abstract public function removeState($synckey = null, $devId = null);
-
-    /**
-     * Return the heartbeat interval, or zero if we have no existing state
-     *
-     * @return integer  The hearbeat interval, or zero if not found.
-     * @throws Horde_ActiveSync_Exception
-     */
-    abstract public function getHeartbeatInterval();
-
-    /**
-     * Set the device's heartbeat interval
-     *
-     * @param integer $heartbeat  The interval (in seconds).
-     */
-    abstract public function setHeartbeatInterval($heartbeat);
+    abstract public function removeState(array $options);
 
     /**
      * List all devices that we know about.
@@ -626,4 +576,48 @@ abstract class Horde_ActiveSync_State_Base
      */
     abstract public function getLastSyncTimestamp();
 
+    /**
+     * Return the sync cache.
+     *
+     * @param string $devid  The device id.
+     * @param string $user   The user id.
+     *
+     * @return array  The current sync cache for the user/device combination.
+     * @throws Horde_ActiveSync_Exception
+     */
+    abstract public function getSyncCache($devid, $user);
+
+    /**
+     * Save the provided sync_cache.
+     *
+     * @param array $cache   The cache to save.
+     * @param string $devid  The device id.
+     * @param string $user   The userid.
+     *
+     * @throws Horde_ActiveSync_Exception
+     */
+    abstract public function saveSyncCache(array $cache, $devid, $user);
+
+    /**
+     * Delete a complete sync cache
+     *
+     * @param string $devid  The device id
+     * @param string $user   The user name.
+     *
+     * @throws Horde_ActiveSync_Exception
+     */
+    abstract public function deleteSyncCache($devid, $user);
+
+    /**
+     * Check and see that we didn't already see the incoming change from the PIM.
+     * This would happen e.g., if the PIM failed to receive the server response
+     * after successfully importing new messages.
+     *
+     * @param string $id  The client id sent during message addition.
+     *
+     * @return string The UID for the given clientid, null if none found.
+     * @throws Horde_ActiveSync_Exception
+     */
+     abstract public function isDuplicatePIMAddition($id);
+
 }
\ No newline at end of file
